# -*- encoding: utf-8 -*-
#ライセンス
require 'spec_helper'

describe SystemController do
  before do
  end

  describe 'システム初期化要請に於いて' do
    context '要請の必要がないとき' do
      before do
        #要請の必要がない=管理者登録済み かつ ライセンス登録済み
        FactoryGirl.create :admin
        @sp = FactoryGirl.create :system_picture
        @lg = FactoryGirl.create :license_group
        @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
      end
      it 'リクエスト通りのページを開く' do
        get :index
        response.should_not redirect_to 'system/start'
      end
      context 'userがサインインしているなら' do
        before do
          @user = FactoryGirl.create( :user_yas)
          @author = @user.author
          sign_in @user
        end
        it 'userにサインイン中のアカウントをセットしている' do
          get :index
          assigns(:user).should eq @user
        end
        it 'authorに作家をセットしている' do
          get :index
          assigns(:author).should eq @author
        end
        it 'artistに絵師をセットしている' do
          @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
          get :index
          assigns(:artist).should eq @artist
        end
        it 'ただし、絵師登録されてなければ新規の絵師をセットしている' do
          get :index
          assigns(:artist).should be_a_new(Artist)
        end
      end
      context 'userがサインインしていないなら' do
        before do
          @user = FactoryGirl.create( :user_yas)
          @author = @user.author
          @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
        end
        it 'userが空になっている' do
          get :index
          assigns(:user).should eq nil
        end
        it 'authorが空になっている' do
          get :index
          assigns(:author).should eq nil
        end
        it 'artistが空になっている' do
          get :index
          assigns(:artist).should eq nil
        end
      end
    end
    #要請の必要がある=管理者がいない または ライセンスがない
    context '管理者がいないとき' do
      before do
        @sp = FactoryGirl.create :system_picture
        @lg = FactoryGirl.create :license_group
        @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
      end
      context '本機能を開こうとしているなら' do
        it '初期化要請ページに遷移する' do
          get :index
          response.should redirect_to '/system/start'
        end
        it 'ステータスコード302 Foundを返す' do
          get :index
          response.status.should eq 302
        end
      end
      context '初期化要請ページを開こうとしているなら' do
        it '遷移しない' do
          get :start
          response.should_not redirect_to 'system/start'
        end
        it 'ステータスOkを返す' do
          get :start
          response.status.should eq 200
        end
        it '初期化要請ページを描画する' do
          get :start
          response.should render_template("system/start")
        end
      end
    end
    context 'ライセンスがないとき' do
      before do
        FactoryGirl.create :admin
      end
      context '本機能を開こうとしているなら' do
        it '初期化要請ページに遷移する' do
          get :index
          response.should redirect_to '/system/start'
        end
        it 'ステータスコード302 Foundを返す' do
          get :index
          response.status.should eq 302
        end
      end
      context '初期化要請ページを開こうとしているなら' do
        it '遷移しない' do
          get :start
          response.should_not redirect_to 'system/start'
        end
        it 'ステータスOkを返す' do
          get :start
          response.status.should eq 200
        end
        it '初期化要請ページを描画する' do
          get :start
          response.should render_template("system/start")
        end
      end
    end
  end
end
