# -*- encoding: utf-8 -*-
require 'spec_helper'
#ユーザホーム

describe HomeController do
  before do
    FactoryGirl.create :admin
    @sp = FactoryGirl.create :system_picture
    @lg = FactoryGirl.create :license_group
    @license = FactoryGirl.create :license, :license_group_id => @lg.id, :system_picture_id => @sp.id
    @user = FactoryGirl.create( :user_yas)
    @author = @user.author
    @artist = FactoryGirl.create :artist_yas, :author_id => @author.id
    @color = FactoryGirl.create :color
    @op = FactoryGirl.create :original_picture, :artist_id => @artist.id
    @p = FactoryGirl.create :picture, :original_picture_id => @op.id, :license_id => @license.id, :artist_id => @artist.id
    @rp = FactoryGirl.create :resource_picture, :artist_id => @artist.id, :license_id => @license.id, :original_picture_id => @op.id, :picture_id => @p.id
    @panel = FactoryGirl.create :panel, :author_id => @author.id
  end
  
  describe '自分のコミック一覧表示に於いて' do
    before do
      @comic = FactoryGirl.create :comic, :author_id => @author.id
      sign_in @user
      Comic.stub(:mylist).and_return([@comic, @comic, @comic])
    end
    context 'パラメータpageについて' do
      it '@pageに値が入る' do
        get :comic, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :comic
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :comic, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :comic
        assigns(:page_size).should eq Author.default_comic_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :comic, :page_size => 1500
        assigns(:page_size).should eq Author.comic_max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :comic, :page_size => 0
        assigns(:page_size).should eq Author.default_comic_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :comic
        response.should be_success 
      end
      it 'コミックモデルに一覧を問い合わせている' do
        Comic.should_receive(:mylist).exactly(1)
        get :comic
      end
      it '@comicsにリストを取得している' do
        get :comic
        assigns(:comics).should have_at_least(3).items
      end
      context 'html形式' do
        it 'comicテンプレートを描画する' do
          get :comic
          response.should render_template("comic")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :comic, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'コミックモデルにjson一覧出力オプションを問い合わせている' do
          Comic.should_receive(:list_json_opt).exactly(1)
          get :comic, :format => :json
        end
        it 'データがリスト構造になっている' do
          get :comic, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいはコミックっぽいものであって欲しい' do
          get :comic, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("title").should be_true
          json.first.has_key?("visible").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :comic
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :comic
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :comic, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :comic, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
  describe '自分のコマ一覧表示に於いて' do
    before do
      @panel = FactoryGirl.create :panel, :author_id => @author.id
      sign_in @user
      Panel.stub(:mylist).and_return([@panel, @panel, @panel])
    end
    context 'パラメータpageについて' do
      it '@pageに値が入る' do
        get :panel, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :panel
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :panel, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :panel
        assigns(:page_size).should eq Author.default_panel_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :panel, :page_size => 1500
        assigns(:page_size).should eq Author.panel_max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :panel, :page_size => 0
        assigns(:page_size).should eq Author.default_panel_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'コマモデルに一覧を問い合わせている' do
        Panel.should_receive(:mylist).exactly(1)
        get :panel
      end
      it '@panelsにリストを取得している' do
        get :panel
        assigns(:panels).should have_at_least(3).items
      end
      context 'html形式' do
        it 'ステータスコード200 OKを返す' do
          get :panel
          response.should be_success 
        end
        it 'panelテンプレートを描画する' do
          get :panel
          response.should render_template("panel")
        end
      end
      context 'json形式' do
        it 'ステータスコード200 OKを返す' do
          get :panel, :format => :json
          response.should be_success 
        end
        it 'jsonデータを返す' do
          get :panel, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'コマモデルにコマリストのjson出力を問い合わせている' do
          Panel.should_receive(:list_as_json_text).exactly(1)
          get :panel, :format => :json
        end
        it 'データがリスト構造になっている' do
          get :panel, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいはコマっぽいものであって欲しい' do
          get :panel, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("z").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :panel
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :panel
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :panel, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :panel, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
  describe '自分のコマ絵一覧表示に於いて' do
    before do
      @pp = FactoryGirl.create :panel_picture, :panel_id => @panel.id, :picture_id => @p.id,
        :width => @p.width, :height => @p.height
      sign_in @user
      PanelPicture.stub(:mylist).and_return([@pp, @pp, @pp])
    end
    context 'パラメータpageについて' do
      it '@pageに値が入る' do
        get :panel_picture, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :panel_picture
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :panel_picture, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :panel_picture
        assigns(:page_size).should eq Author.default_panel_picture_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :panel_picture, :page_size => 1500
        assigns(:page_size).should eq Author.panel_picture_max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :panel_picture, :page_size => 0
        assigns(:page_size).should eq Author.default_panel_picture_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :panel_picture
        response.should be_success 
      end
      it 'コマ絵モデルに一覧を問い合わせている' do
        PanelPicture.should_receive(:mylist).exactly(1)
        get :panel_picture
      end
      it '@panel_picturesにリストを取得している' do
        get :panel_picture
        assigns(:panel_pictures).should have_at_least(3).items
      end
      context 'html形式' do
        it 'panel_pictureテンプレートを描画する' do
          get :panel_picture
          response.should render_template("panel_picture")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :panel_picture, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'コマ絵モデルにjson一覧出力オプションを問い合わせている' do
          PanelPicture.should_receive(:list_json_opt).exactly(1)
          get :panel_picture, :format => :json
        end
        it 'データがリスト構造になっている' do
          get :panel_picture, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいはコマ絵っぽいものであって欲しい' do
          get :panel_picture, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("link").should be_true
          json.first.has_key?("x").should be_true
          json.first.has_key?("y").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :panel_picture
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :panel_picture
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :panel_picture, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :panel_picture, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
  describe '自分の原画一覧表示に於いて' do
    before do
      sign_in @user
      OriginalPicture.stub(:mylist).and_return([@op, @op, @op])
    end
    context 'パラメータpageについて' do
      it '@pageに値が入る' do
        get :picture, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :picture
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :picture, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :picture
        assigns(:page_size).should eq OriginalPicture.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :picture, :page_size => 1500
        assigns(:page_size).should eq OriginalPicture.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :picture, :page_size => 0
        assigns(:page_size).should eq OriginalPicture.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :picture
        response.should be_success 
      end
      it '原画モデルに一覧を問い合わせている' do
        #原画は他人が使えないので「自分の」リストはない
        OriginalPicture.should_receive(:mylist).exactly(1)
        get :picture
      end
      it '@original_picturesにリストを取得している' do
        get :picture
        assigns(:original_pictures).should have_at_least(3).items
      end
      context 'html形式' do
        it 'pictureテンプレートを描画する' do
          get :picture
          response.should render_template("picture")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :picture, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it '原画モデルにjson一覧出力オプションを問い合わせている' do
          OriginalPicture.should_receive(:list_json_opt).exactly(1)
          get :picture, :format => :json
        end
        it 'データがリスト構造になっている' do
          get :picture, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいは原画っぽいものであって欲しい' do
          get :picture, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("ext").should be_true
          json.first.has_key?("md5").should be_true
          json.first.has_key?("artist_id").should be_true
          json.first.has_key?("width").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :picture
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :picture
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :picture, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :picture, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
    context '作家が絵師でないとき' do
      before do
        Author.any_instance.stub(:artist?).and_return(false)
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :picture
          response.status.should eq 302
        end
        it '絵師登録ページへ遷移する' do
          get :picture
          response.should redirect_to new_artist_path
        end
      end
      context 'json形式' do
        it '例外403 forbiddenを返す' do
          lambda{
            get :picture, :format => :json
          }.should raise_error(ActiveRecord::Forbidden)
        end
      end
    end
  end
  
  describe '自分のコマの色背景一覧表示に於いて' do
    before do
      @pc = FactoryGirl.create :panel_color, :panel_id => @panel.id
      sign_in @user
      PanelColor.stub(:mylist).and_return([@pc, @pc, @pc])
    end
    context 'パラメータpageについて' do
      it '@pageに値が入る' do
        get :panel_color, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :panel_color
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :panel_color, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :panel_color
        assigns(:page_size).should eq PanelColor.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :panel_color, :page_size => 1500
        assigns(:page_size).should eq PanelColor.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :panel_color, :page_size => 0
        assigns(:page_size).should eq PanelColor.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :panel_color
        response.should be_success 
      end
      it 'コマの色背景モデルに一覧を問い合わせている' do
        PanelColor.should_receive(:mylist).exactly(1)
        get :panel_color
      end
      it '@panel_colorsにリストを取得している' do
        get :panel_color
        assigns(:panel_colors).should have_at_least(3).items
      end
      context 'html形式' do
        it 'panel_colorテンプレートを描画する' do
          get :panel_color
          response.should render_template("panel_color")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :panel_color, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'コマの色背景モデルにjson一覧出力オプションを問い合わせている' do
          PanelColor.should_receive(:list_json_opt).exactly(1)
          get :panel_color, :format => :json
        end
        it 'データがリスト構造になっている' do
          get :panel_color, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいはコマの色背景っぽいものであって欲しい' do
          get :panel_color, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("panel_id").should be_true
          json.first.has_key?("code").should be_true
          json.first.has_key?("z").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :panel_color
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :panel_color
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :panel_color, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :panel_color, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
  describe '自分のコマの画像背景一覧表示に於いて' do
    before do
      @gp = FactoryGirl.create :ground_picture, :panel_id => @panel.id, :picture_id => @p.id
      sign_in @user
      GroundPicture.stub(:mylist).and_return([@gp, @gp, @gp])
    end
    context 'パラメータpageについて' do
      it '@pageに値が入る' do
        get :ground_picture, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :ground_picture
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :ground_picture, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :ground_picture
        assigns(:page_size).should eq GroundPicture.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :ground_picture, :page_size => 1500
        assigns(:page_size).should eq GroundPicture.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :ground_picture, :page_size => 0
        assigns(:page_size).should eq GroundPicture.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :ground_picture
        response.should be_success 
      end
      it 'コマの画像背景モデルに一覧を問い合わせている' do
        GroundPicture.should_receive(:mylist).exactly(1)
        get :ground_picture
      end
      it '@ground_picturesにリストを取得している' do
        get :ground_picture
        assigns(:ground_pictures).should have_at_least(3).items
      end
      context 'html形式' do
        it 'ground_pictureテンプレートを描画する' do
          get :ground_picture
          response.should render_template("ground_picture")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :ground_picture, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'コマの画像背景モデルにjson一覧出力オプションを問い合わせている' do
          GroundPicture.should_receive(:list_json_opt).exactly(1)
          get :ground_picture, :format => :json
        end
        it 'データがリスト構造になっている' do
          get :ground_picture, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいはコマの画像背景っぽいものであって欲しい' do
          get :ground_picture, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("panel_id").should be_true
          json.first.has_key?("picture_id").should be_true
          json.first.has_key?("z").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :ground_picture
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :ground_picture
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :ground_picture, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :ground_picture, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
  describe '自分の間接背景一覧表示に於いて' do
    before do
      @gc = FactoryGirl.create :ground_color
      sign_in @user
      GroundColor.stub(:mylist).and_return([@gc, @gc, @gc])
    end
    context 'パラメータpageについて' do
      it '@pageに値が入る' do
        get :ground_color, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :ground_color
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :ground_color, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :ground_color
        assigns(:page_size).should eq GroundColor.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :ground_color, :page_size => 1500
        assigns(:page_size).should eq GroundColor.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :ground_color, :page_size => 0
        assigns(:page_size).should eq GroundColor.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :ground_color
        response.should be_success 
      end
      it '間接背景モデルに一覧を問い合わせている' do
        GroundColor.should_receive(:mylist).exactly(1)
        get :ground_color
      end
      it '@ground_colorsにリストを取得している' do
        get :ground_color
        assigns(:ground_colors).should have_at_least(3).items
      end
      context 'html形式' do
        it 'ground_colorテンプレートを描画する' do
          get :ground_color
          response.should render_template("ground_color")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :ground_color, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it '間接背景モデルにjson一覧出力オプションを問い合わせている' do
          GroundColor.should_receive(:list_json_opt).exactly(1)
          get :ground_color, :format => :json
        end
        it 'データがリスト構造になっている' do
          get :ground_color, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいは間接背景っぽいものであって欲しい' do
          get :ground_color, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("panel_id").should be_true
          json.first.has_key?("color_id").should be_true
          json.first.has_key?("z").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :ground_color
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :ground_color
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :ground_color, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :ground_color, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
end
